@@ -4,8 +4,9 @@ import engineio
 import six
 
 from . import base_manager
-from . import packet
+from . import exceptions
 from . import namespace
+from . import packet
 
 default_logger = logging.getLogger('socketio.server')
 
@@ -33,9 +34,20 @@ class Server(object):
                  packets. Custom json modules must have ``dumps`` and ``loads``
                  functions that are compatible with the standard library
                  versions.
-    :param async_handlers: If set to ``True``, event handlers are executed in
-                           separate threads. To run handlers synchronously,
-                           set to ``False``. The default is ``False``.
+    :param async_handlers: If set to ``True``, event handlers for a client are
+                           executed in separate threads. To run handlers for a
+                           client synchronously, set to ``False``. The default
+                           is ``True``.
+    :param always_connect: When set to ``False``, new connections are
+                           provisory until the connect handler returns
+                           something other than ``False``, at which point they
+                           are accepted. When set to ``True``, connections are
+                           immediately accepted, and then if the connect
+                           handler returns ``False`` a disconnect is issued.
+                           Set to ``True`` if you need to emit events from the
+                           connect handler and your client is confused when it
+                           receives events before the connection acceptance.
+                           In any other case use the default of ``False``.
     :param kwargs: Connection parameters for the underlying Engine.IO server.
 
     The Engine.IO configuration supports the following settings:
@@ -77,7 +89,8 @@ class Server(object):
                             ``False``. The default is ``False``.
     """
     def __init__(self, client_manager=None, logger=False, binary=False,
-                 json=None, async_handlers=False, **kwargs):
+                 json=None, async_handlers=True, always_connect=False,
+                 **kwargs):
         engineio_options = kwargs
         engineio_logger = engineio_options.pop('engineio_logger', None)
         if engineio_logger is not None:
@@ -117,6 +130,7 @@ class Server(object):
         self.manager_initialized = False
 
         self.async_handlers = async_handlers
+        self.always_connect = always_connect
 
         self.async_mode = self.eio.async_mode
 
@@ -224,11 +238,11 @@ class Server(object):
         namespace = namespace or '/'
         self.logger.info('emitting event "%s" to %s [%s]', event,
                          room or 'all', namespace)
-        self.manager.emit(event, data, namespace, room, skip_sid, callback,
-                          **kwargs)
+        self.manager.emit(event, data, namespace, room=room,
+                          skip_sid=skip_sid, callback=callback, **kwargs)
 
     def send(self, data, room=None, skip_sid=None, namespace=None,
-             callback=None, **kwargs):
+             callback=None, wait=False, timeout=60, **kwargs):
         """Send a message to one or more connected clients.
 
         This function emits an event with the name ``'message'``. Use
@@ -261,8 +275,51 @@ class Server(object):
                              to always leave this parameter with its default
                              value of ``False``.
         """
-        self.emit('message', data, room, skip_sid, namespace, callback,
-                  **kwargs)
+        self.emit('message', data=data, room=room, skip_sid=skip_sid,
+                  namespace=namespace, callback=callback, **kwargs)
+
+    def call(self, event, data=None, sid=None, namespace=None, timeout=60,
+             **kwargs):
+        """Emit a custom event to a client and wait for the response.
+
+        :param event: The event name. It can be any string. The event names
+                      ``'connect'``, ``'message'`` and ``'disconnect'`` are
+                      reserved and should not be used.
+        :param data: The data to send to the client or clients. Data can be of
+                     type ``str``, ``bytes``, ``list`` or ``dict``. If a
+                     ``list`` or ``dict``, the data will be serialized as JSON.
+        :param sid: The session ID of the recipient client.
+        :param namespace: The Socket.IO namespace for the event. If this
+                          argument is omitted the event is emitted to the
+                          default namespace.
+        :param timeout: The waiting timeout. If the timeout is reached before
+                        the client acknowledges the event, then a
+                        ``TimeoutError`` exception is raised.
+        :param ignore_queue: Only used when a message queue is configured. If
+                             set to ``True``, the event is emitted to the
+                             client directly, without going through the queue.
+                             This is more efficient, but only works when a
+                             single server process is used. It is recommended
+                             to always leave this parameter with its default
+                             value of ``False``.
+        """
+        if not self.async_handlers:
+            raise RuntimeError(
+                'Cannot use call() when async_handlers is False.')
+        callback_event = self.eio.create_event()
+        callback_args = []
+
+        def event_callback(*args):
+            callback_args.append(args)
+            callback_event.set()
+
+        self.emit(event, data=data, room=sid, namespace=namespace,
+                  callback=event_callback, **kwargs)
+        if not callback_event.wait(timeout=timeout):
+            raise exceptions.TimeoutError()
+        return callback_args[0] if len(callback_args[0]) > 1 \
+            else callback_args[0][0] if len(callback_args[0]) == 1 \
+                else None
 
     def enter_room(self, sid, room, namespace=None):
         """Enter a room.
@@ -330,8 +387,8 @@ class Server(object):
         is used.
         """
         namespace = namespace or '/'
-        with self.eio.session(sid) as eio_session:
-            return eio_session.setdefault(namespace, {})
+        eio_session = self.eio.get_session(sid)
+        return eio_session.setdefault(namespace, {})
 
     def save_session(self, sid, session, namespace=None):
         """Store the user session for a client.
@@ -342,8 +399,8 @@ class Server(object):
                           the default namespace is used.
         """
         namespace = namespace or '/'
-        with self.eio.session(sid) as eio_session:
-            eio_session[namespace] = session
+        eio_session = self.eio.get_session(sid)
+        eio_session[namespace] = session
 
     def session(self, sid, namespace=None):
         """Return the user session for a client with context manager syntax.
@@ -485,15 +542,30 @@ class Server(object):
         """Handle a client connection request."""
         namespace = namespace or '/'
         self.manager.connect(sid, namespace)
-        if self._trigger_event('connect', namespace, sid,
-                               self.environ[sid]) is False:
-            self.manager.disconnect(sid, namespace)
-            self._send_packet(sid, packet.Packet(packet.ERROR,
+        if self.always_connect:
+            self._send_packet(sid, packet.Packet(packet.CONNECT,
                                                  namespace=namespace))
+        fail_reason = None
+        try:
+            success = self._trigger_event('connect', namespace, sid,
+                                          self.environ[sid])
+        except exceptions.ConnectionRefusedError as exc:
+            fail_reason = exc.error_args
+            success = False
+
+        if success is False:
+            if self.always_connect:
+                self.manager.pre_disconnect(sid, namespace)
+                self._send_packet(sid, packet.Packet(
+                    packet.DISCONNECT, data=fail_reason, namespace=namespace))
+            self.manager.disconnect(sid, namespace)
+            if not self.always_connect:
+                self._send_packet(sid, packet.Packet(
+                    packet.ERROR, data=fail_reason, namespace=namespace))
             if sid in self.environ:  # pragma: no cover
                 del self.environ[sid]
             return False
-        else:
+        elif not self.always_connect:
             self._send_packet(sid, packet.Packet(packet.CONNECT,
                                                  namespace=namespace))
 
